@@ -15,11 +15,8 @@ source $BIN/activate
 set -u
 
 $BIN/pip -v install --upgrade pip setuptools wheel
-$BIN/pip install nose
-
-# run tests with nose
 $BIN/pip -v install --force .
-$BIN/nosetests -s -v -P python/tests
 
-# run tests through setup.py; also builds documentation
-$BIN/python setup.py test
+pushd python/tests
+$BIN/python -m unittest -v 
+popd
