@@ -16,7 +16,7 @@ namespace H5 = HighFive;
 namespace bbp {
 namespace sonata {
 
-// KeyType will be NodeID for somas report and pair<NodeID, ElementID> for elements report
+// KeyType will be NodeID for somas report and CompartmentID for elements report
 template <typename KeyType>
 struct SONATA_API DataFrame {
     using DataType = std::vector<KeyType>;
@@ -51,6 +51,11 @@ class SONATA_API SpikeReader
             by_time = 2,
         };
 
+        /**
+         * Return (tstart, tstop) of the population
+         */
+        std::tuple<double, double> getTimes() const;
+
         /**
          * Return reports for this population.
          */
@@ -97,9 +102,6 @@ template <typename KeyType>
 class SONATA_API ReportReader
 {
   public:
-    using Range = std::pair<uint64_t, uint64_t>;
-    using Ranges = std::vector<Range>;
-
     class Population
     {
       public:
@@ -122,8 +124,27 @@ class SONATA_API ReportReader
          * Return true if the data is sorted.
          */
         bool getSorted() const;
+
+        /**
+         * Return all the node ids.
+         */
         std::vector<NodeID> getNodeIds() const;
 
+        /**
+         * Return the ElementIds for the passed Node.
+         * The return type will depend on the report reader:
+         * - For Soma report reader, the return value will be the Node ID to which the report
+         *   value belongs to.
+         * - For Element/full compartment readers, the return value will be an array with 2
+         *   elements, the first element is the Node ID and the second element is the 
+         *   compartment ID of the given Node.
+         *
+         * \param node_ids limit the report to the given selection. If nullptr, all nodes in the
+         * report are used
+         */
+        typename DataFrame<KeyType>::DataType getNodeIdElementIdMapping(
+            const nonstd::optional<Selection>& node_ids = nonstd::nullopt) const;
+
         /**
          * \param node_ids limit the report to the given selection.
          * \param tstart return voltages occurring on or after tstart. tstart=nonstd::nullopt
@@ -137,11 +158,27 @@ class SONATA_API ReportReader
                                const nonstd::optional<size_t>& tstride = nonstd::nullopt) const;
 
       private:
+        struct NodeIdElementLayout {
+            typename DataFrame<KeyType>::DataType ids;
+            Selection::Ranges node_ranges;
+            Selection::Range min_max_range;
+        };
+
         Population(const H5::File& file, const std::string& populationName);
         std::pair<size_t, size_t> getIndex(const nonstd::optional<double>& tstart,
                                            const nonstd::optional<double>& tstop) const;
+        /**
+         * Return the element IDs for the given selection, alongside the filtered node pointers
+         * and the range of positions where they fit in the file. This latter two are necessary
+         * for performance to understand how and where to retrieve the data from storage.
+         *
+         * \param node_ids limit the report to the given selection. If nullptr, all nodes in the
+         * report are used
+         */
+        NodeIdElementLayout getNodeIdElementLayout(
+            const nonstd::optional<Selection>& node_ids = nonstd::nullopt) const;
 
-        std::map<NodeID, Range> nodes_pointers_;
+        std::map<NodeID, Selection::Range> node_ranges_;
         H5::Group pop_group_;
         std::vector<NodeID> nodes_ids_;
         double tstart_, tstop_, tstep_;
@@ -170,7 +207,7 @@ class SONATA_API ReportReader
 };
 
 using SomaReportReader = ReportReader<NodeID>;
-using ElementReportReader = ReportReader<std::pair<NodeID, ElementID>>;
+using ElementReportReader = ReportReader<CompartmentID>;
 
 }  // namespace sonata
 }  // namespace bbp
