@@ -3,6 +3,7 @@
 #include <pybind11/stl.h>
 
 #include <bbp/sonata/common.h>
+#include <bbp/sonata/config.h>
 #include <bbp/sonata/edges.h>
 #include <bbp/sonata/node_sets.h>
 #include <bbp/sonata/nodes.h>
@@ -329,12 +330,14 @@ void bindReportReader(py::module& m, const std::string& prefix) {
     py::class_<DataFrame<KeyType>>(m,
                                    (prefix + "DataFrame").c_str(),
                                    "A container of raw reporting data, compatible with Pandas")
-        .def_readonly("ids", &DataFrame<KeyType>::ids)
-
-        // .data and .time members are owned by this c++ object. We can't do std::move.
-        // To avoid copies we must declare the owner of the data as the current python
+        // .ids, .data and .time members are owned by the c++ object. We can't do std::move.
+        // To avoid copies, we must declare the owner of the data as the current Python
         // object. Numpy will adjust owner reference count according to returned arrays
         // clang-format off
+        .def_property_readonly("ids", [](const DataFrame<KeyType>& dframe) {
+            std::array<ssize_t, 1> dims { ssize_t(dframe.ids.size()) };
+            return managedMemoryArray(dframe.ids.data(), dims, dframe);
+        })
         .def_property_readonly("data", [](const DataFrame<KeyType>& dframe) {
             std::array<ssize_t, 2> dims {0l, ssize_t(dframe.ids.size())};
             if (dims[1] > 0) {
@@ -361,6 +364,14 @@ void bindReportReader(py::module& m, const std::string& prefix) {
         .def("get_node_ids",
              &ReportType::Population::getNodeIds,
              "Return the list of nodes ids for this population")
+        .def(
+            "get_node_id_element_id_mapping",
+            [](const typename ReportType::Population& population,
+               const nonstd::optional<Selection>& selection) {
+                return population.getNodeIdElementIdMapping(selection);
+            },
+            DOC_REPORTREADER_POP(getNodeIdElementIdMapping),
+            "selection"_a = nonstd::nullopt)
         .def_property_readonly("sorted",
                                &ReportType::Population::getSorted,
                                DOC_REPORTREADER_POP(getSorted))
@@ -473,11 +484,83 @@ PYBIND11_MODULE(_libsonata, m) {
 
     py::class_<NodeSets>(m, "NodeSets", "")
         .def(py::init<const std::string&>())
-        .def_static("from_file", &NodeSets::fromFile)
+        .def_static("from_file", [](py::object path) { return NodeSets::fromFile(py::str(path)); })
         .def_property_readonly("names", &NodeSets::names, DOC_NODESETS(names))
         .def("materialize", &NodeSets::materialize, DOC_NODESETS(materialize))
         .def("toJSON", &NodeSets::toJSON, DOC_NODESETS(toJSON));
 
+    py::class_<PopulationProperties>(m,
+                                     "PopulationProperties",
+                                     "Stores population-specific network information")
+        .def_readonly("type", &PopulationProperties::type, "Population type")
+        .def_readonly("biophysical_neuron_models_dir",
+                      &PopulationProperties::biophysicalNeuronModelsDir,
+                      "Path to the template HOC files defining the E-Mode")
+        .def_readonly("morphologies_dir",
+                      &PopulationProperties::morphologiesDir,
+                      "Path to the directory containing the morphologies")
+        .def_readonly("alternate_morphology_formats",
+                      &PopulationProperties::alternateMorphologyFormats,
+                      "Path to the directory containing the morphologies");
+
+    py::class_<CircuitConfig>(m, "CircuitConfig", "")
+        .def(py::init<const std::string&, const std::string&>())
+        .def_static("from_file",
+                    [](py::object path) { return CircuitConfig::fromFile(py::str(path)); })
+        .def_property_readonly("node_sets_path", &CircuitConfig::getNodeSetsPath)
+        .def_property_readonly("node_populations", &CircuitConfig::listNodePopulations)
+        .def("node_population", &CircuitConfig::getNodePopulation)
+        .def_property_readonly("edge_populations", &CircuitConfig::listEdgePopulations)
+        .def("edge_population", &CircuitConfig::getEdgePopulation)
+        .def("node_population_properties", &CircuitConfig::getNodePopulationProperties, "name"_a)
+        .def("edge_population_properties", &CircuitConfig::getEdgePopulationProperties, "name"_a)
+        .def_property_readonly("expanded_json", &CircuitConfig::getExpandedJSON);
+
+    py::class_<SimulationConfig::Run>(m,
+                                      "Run",
+                                      "Stores parameters defining global simulation settings")
+        .def_readonly("tstop",
+                      &SimulationConfig::Run::tstop,
+                      "Biological simulation end time in milliseconds")
+        .def_readonly("dt",
+                      &SimulationConfig::Run::dt,
+                      "Simulation integration step in milliseconds");
+
+    py::class_<SimulationConfig::Output>(m,
+                                         "Output",
+                                         "Stores overriden parameters of simulation output")
+        .def_readonly("output_dir",
+                      &SimulationConfig::Output::outputDir,
+                      "Simulation output directory")
+        .def_readonly("spikes_file",
+                      &SimulationConfig::Output::spikesFile,
+                      "Spike report filename");
+
+    py::class_<SimulationConfig::Report>(m, "Report", "List of parameters of a report")
+        .def_readonly("cells", &SimulationConfig::Report::cells, "Node sets on which to report")
+        .def_readonly("type",
+                      &SimulationConfig::Report::type,
+                      "Report type. Possible values are 'compartment', 'summation', 'synapse")
+        .def_readonly("dt",
+                      &SimulationConfig::Report::dt,
+                      "Interval between reporting steps in milliseconds")
+        .def_readonly("start_time",
+                      &SimulationConfig::Report::startTime,
+                      "Time to start reporting, in milliseconds")
+        .def_readonly("end_time",
+                      &SimulationConfig::Report::endTime,
+                      "Time to stop reporting in milliseconds")
+        .def_readonly("file_name", &SimulationConfig::Report::fileName, "Report file name");
+
+    py::class_<SimulationConfig>(m, "SimulationConfig", "")
+        .def(py::init<const std::string&, const std::string&>())
+        .def_static("from_file",
+                    [](py::object path) { return SimulationConfig::fromFile(py::str(path)); })
+        .def_property_readonly("base_path", &SimulationConfig::getBasePath)
+        .def_property_readonly("json", &SimulationConfig::getJSON)
+        .def_property_readonly("run", &SimulationConfig::getRun)
+        .def_property_readonly("output", &SimulationConfig::getOutput)
+        .def("report", &SimulationConfig::getReport, "name"_a);
 
     bindPopulationClass<EdgePopulation>(
         m, "EdgePopulation", "Collection of edges with attributes and connectivity index")
@@ -584,7 +667,10 @@ PYBIND11_MODULE(_libsonata, m) {
                     return "by_time";
                 return "none";
             },
-            DOC_SPIKEREADER_POP(getSorting));
+            DOC_SPIKEREADER_POP(getSorting))
+        .def_property_readonly("times",
+                               &SpikeReader::Population::getTimes,
+                               DOC_SPIKEREADER_POP(getTimes));
     py::class_<SpikeReader>(m, "SpikeReader", "Used to read spike files")
         .def(py::init([](py::object h5_filepath) { return SpikeReader(py::str(h5_filepath)); }),
              "h5_filepath"_a)
@@ -594,7 +680,7 @@ PYBIND11_MODULE(_libsonata, m) {
         .def("__getitem__", &SpikeReader::openPopulation);
 
     bindReportReader<SomaReportReader, NodeID>(m, "Soma");
-    bindReportReader<ElementReportReader, std::pair<NodeID, uint32_t>>(m, "Element");
+    bindReportReader<ElementReportReader, CompartmentID>(m, "Element");
 
     py::register_exception<SonataError>(m, "SonataError");
 }
