@@ -1,3 +1,4 @@
+import json
 import os
 import pathlib
 import unittest
@@ -10,6 +11,7 @@ from libsonata import (EdgeStorage, NodeStorage,
                        SomaReportReader, SomaReportPopulation,
                        ElementReportReader, ElementReportPopulation,
                        NodeSets,
+                       CircuitConfig, SimulationConfig
                        )
 
 
@@ -289,6 +291,10 @@ class TestSpikePopulation(unittest.TestCase):
 
         self.assertEqual(len(self.test_obj['All'].get(node_ids=[])), 0)
 
+    def test_getTimes_from_population(self):
+        self.assertEqual(self.test_obj['All'].times, (0.1, 1.3))
+
+
 class TestSomaReportPopulation(unittest.TestCase):
     def setUp(self):
         path = os.path.join(PATH, "somas.h5")
@@ -314,15 +320,23 @@ class TestSomaReportPopulation(unittest.TestCase):
 
         sel = self.test_obj['All'].get(node_ids=[13, 14], tstart=0.8, tstop=1.0)
         self.assertEqual(len(sel.times), 2)  # Number of timestamp (0.8 and 0.9)
-        self.assertEqual(list(sel.ids), [13, 14])
+        keys = sel.ids
+        keys_ref = np.asarray([13, 14])
+        self.assertTrue((keys == keys_ref).all())
         np.testing.assert_allclose(sel.data, [[13.8, 14.8], [13.9, 14.9]])
 
         sel_all = self.test_obj['All'].get()
-        self.assertEqual(sel_all.ids, [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20])
+        keys_all = sel_all.ids
+        keys_all_ref = np.asarray([1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20])
+        self.assertTrue((keys_all == keys_all_ref).all())
 
         sel_empty = self.test_obj['All'].get(node_ids=[])
         np.testing.assert_allclose(sel_empty.data, np.empty(shape=(0, 0)))
 
+    def test_get_node_id_element_id_mapping(self):
+        self.assertEqual(self.test_obj['All'].get_node_id_element_id_mapping([[3, 5]]),
+                         [3, 4])
+
 
 class TestElementReportPopulation(unittest.TestCase):
     def setUp(self):
@@ -353,10 +367,9 @@ class TestElementReportPopulation(unittest.TestCase):
         self.assertEqual(len(self.test_obj['All'].get(tstride=2).times), 10)  # Should be the same
         self.assertEqual(len(self.test_obj['All'].get().ids), 100)
         sel = self.test_obj['All'].get(node_ids=[13, 14], tstart=0.8, tstop=1.2)
-        keys = list(sel.ids)
-        keys.sort()
-        self.assertEqual(keys, [(13, 30), (13, 30), (13, 31), (13, 31), (13, 32), (14, 32), (14, 33), (14, 33), (14, 34), (14, 34)])
-
+        keys = sel.ids
+        keys_ref = np.asarray([(13, 30), (13, 30), (13, 31), (13, 31), (13, 32), (14, 32), (14, 33), (14, 33), (14, 34), (14, 34)])
+        self.assertTrue((keys == keys_ref).all())
         self.assertEqual(len(self.test_obj['All'].get(node_ids=[]).data), 0)
         self.assertEqual(len(self.test_obj['All'].get(node_ids=[]).times), 0)
         self.assertEqual(len(self.test_obj['All'].get(node_ids=[]).ids), 0)
@@ -376,6 +389,10 @@ class TestElementReportPopulation(unittest.TestCase):
         np.testing.assert_allclose(self.test_obj['All'].get(node_ids=[3, 4], tstride=4).data[2],
                                    [81.0, 81.1, 81.2, 81.3, 81.4, 81.5, 81.6, 81.7, 81.8, 81.9], 1e-6, 0)
 
+    def test_get_node_id_element_id_mapping(self):
+        self.assertEqual(self.test_obj['All'].get_node_id_element_id_mapping([[3, 5]]),
+                         [[3, 5], [3, 5], [3, 6], [3, 6], [3, 7], [4, 7], [4, 8], [4, 8], [4, 9], [4, 9]])
+
 
 class TestNodePopulationFailure(unittest.TestCase):
     def test_CorrectStructure(self):
@@ -462,6 +479,15 @@ class TestNodePopulationNodeSet(unittest.TestCase):
               "model_type": "point",
               "node_id": [1, 2, 3, 5, 7, 9]
           },
+          "power_number_test": {
+              "numeric_attribute_gt": { "$gt": 3 },
+              "numeric_attribute_lt": { "$lt": 3 },
+              "numeric_attribute_gte": { "$gte": 3 },
+              "numeric_attribute_lte": { "$lte": 3 }
+          },
+          "power_regex_test": {
+              "string_attr": { "$regex": "^[s][o]me value$" }
+          },
           "combined": ["bio_layer45", "V1_point_prime"]
         }'''
         new = NodeSets(j).toJSON()
@@ -481,6 +507,75 @@ def test_path_ctor():
     SpikeReader(path / 'spikes.h5')
     SomaReportReader(path / 'somas.h5')
     ElementReportReader(path / 'elements.h5')
+    NodeSets.from_file(path / 'node_sets.json')
+    CircuitConfig.from_file(path / 'config/circuit_config.json')
+
+
+class TestCircuitConfig(unittest.TestCase):
+    def setUp(self):
+        self.config = CircuitConfig.from_file(os.path.join(PATH, 'config/circuit_config.json'))
+
+    def test_basic(self):
+        self.assertEqual(self.config.node_sets_path,
+                         os.path.abspath(os.path.join(PATH, 'config/node_sets.json')))
+
+        self.assertEqual(self.config.node_populations,
+                         {'nodes-A', 'nodes-B'})
+        self.assertEqual(self.config.node_population('nodes-A').name, 'nodes-A')
+
+        self.assertEqual(self.config.edge_populations,
+                         {'edges-AB', 'edges-AC'})
+        self.assertEqual(self.config.edge_population('edges-AB').name, 'edges-AB')
+
+    def test_expanded_json(self):
+        config = json.loads(self.config.expanded_json)
+        self.assertEqual(config['components']['biophysical_neuron_models_dir'],
+                         'biophysical_neuron_models')
+        self.assertEqual(config['networks']['nodes'][0]['node_types_file'],
+                         None)
+        self.assertEqual(config['networks']['nodes'][0]['nodes_file'],
+                         '../nodes1.h5')
+
+    def test_get_population_properties(self):
+        node_prop = self.config.node_population_properties('nodes-A')
+        self.assertEqual(node_prop.type, 'biophysical')
+        self.assertTrue(node_prop.morphologies_dir.endswith('morphologies'))
+        self.assertTrue(node_prop.biophysical_neuron_models_dir.endswith('biophysical_neuron_models'))
+        self.assertEqual(node_prop.alternate_morphology_formats, {})
+
+        edge_prop = self.config.edge_population_properties('edges-AC')
+        self.assertEqual(edge_prop.type, 'chemical_synapse')
+        self.assertTrue(edge_prop.morphologies_dir.endswith('morphologies'))
+        self.assertTrue(edge_prop.biophysical_neuron_models_dir.endswith('biophysical_neuron_models'))
+        self.assertEqual(edge_prop.alternate_morphology_formats, {})
+
+
+class TestSimulationConfig(unittest.TestCase):
+    def setUp(self):
+        self.config = SimulationConfig.from_file(
+                        os.path.join(PATH, 'config/simulation_config.json'))
+
+    def test_basic(self):
+        self.assertEqual(self.config.base_path, os.path.abspath(os.path.join(PATH, 'config')))
+
+        self.assertEqual(self.config.run.tstop, 1000)
+        self.assertTrue(abs(self.config.run.dt - 0.025) < 0.01)
+
+        self.assertEqual(self.config.output.output_dir,
+                         os.path.abspath(os.path.join(PATH, 'config/output')))
+        self.assertEqual(self.config.output.spikes_file, 'out.h5')
+
+        self.assertEqual(self.config.report('soma').cells, 'Mosaic')
+        self.assertEqual(self.config.report('soma').type, 'compartment')
+        self.assertTrue(abs(self.config.report('compartment').dt - 0.1) < 0.01)
+        self.assertEqual(self.config.report('axonal_comp_centers').start_time, 0)
+        self.assertEqual(self.config.report('axonal_comp_centers').file_name,
+                         os.path.abspath(os.path.join(PATH, 'config/axon_centers.h5')))
+        self.assertEqual(self.config.report('cell_imembrane').end_time, 500)
+
+    def test_json(self):
+        temp_config = json.loads(self.config.json)
+        self.assertEqual(temp_config['run']['tstop'], 1000)
 
 
 if __name__ == '__main__':
