@@ -15,6 +15,7 @@ HIGHFIVE_REGISTER_TYPE(bbp::sonata::SpikeReader::Population::Sorting, create_enu
 
 namespace {
 
+using bbp::sonata::CompartmentID;
 using bbp::sonata::ElementID;
 using bbp::sonata::NodeID;
 using bbp::sonata::Selection;
@@ -86,7 +87,7 @@ NodeID make_key(NodeID node_id, ElementID /* element_id */) {
 }
 
 template <>
-std::pair<NodeID, ElementID> make_key(NodeID node_id, ElementID element_id) {
+CompartmentID make_key(NodeID node_id, ElementID element_id) {
     return {node_id, element_id};
 }
 
@@ -111,6 +112,10 @@ auto SpikeReader::openPopulation(const std::string& populationName) const -> con
     return populations_.at(populationName);
 }
 
+std::tuple<double, double> SpikeReader::Population::getTimes() const {
+    return std::tie(tstart_, tstop_);
+}
+
 Spikes SpikeReader::Population::get(const nonstd::optional<Selection>& node_ids,
                                     const nonstd::optional<double>& tstart,
                                     const nonstd::optional<double>& tstop) const {
@@ -226,8 +231,8 @@ ReportReader<T>::Population::Population(const H5::File& file, const std::string&
         mapping_group.getDataSet("index_pointers").read(index_pointers);
 
         for (size_t i = 0; i < nodes_ids_.size(); ++i) {
-            nodes_pointers_.emplace(nodes_ids_[i],
-                                    std::make_pair(index_pointers[i], index_pointers[i + 1]));
+            node_ranges_.emplace(nodes_ids_[i],
+                                 std::make_pair(index_pointers[i], index_pointers[i + 1]));
         }
 
         {  // Get times
@@ -278,6 +283,85 @@ std::vector<NodeID> ReportReader<T>::Population::getNodeIds() const {
     return nodes_ids_;
 }
 
+template <typename T>
+typename ReportReader<T>::Population::NodeIdElementLayout
+ReportReader<T>::Population::getNodeIdElementLayout(
+    const nonstd::optional<Selection>& node_ids) const {
+    std::vector<NodeID> concrete_node_ids;
+    NodeIdElementLayout result;
+    result.min_max_range = {std::numeric_limits<uint64_t>::max(),
+                            std::numeric_limits<uint64_t>::min()};
+    size_t element_ids_count = 0;
+
+    // Helper function to update layout, alongside the min / max values and count
+    const auto update_layout = [&](const NodeID& node_id, const Selection::Range& range) {
+        concrete_node_ids.emplace_back(node_id);
+        result.node_ranges.emplace_back(range);
+
+        result.min_max_range.first = std::min(result.min_max_range.first, range.first);
+        result.min_max_range.second = std::max(result.min_max_range.second, range.second);
+        element_ids_count += (range.second - range.first);
+    };
+
+    // Take all nodes if no selection is provided
+    if (!node_ids) {
+        concrete_node_ids.reserve(node_ranges_.size());
+        result.node_ranges.reserve(node_ranges_.size());
+
+        for (const auto& node_range : node_ranges_) {
+            update_layout(node_range.first, node_range.second);
+        }
+    } else if (!node_ids->empty()) {
+        const auto selected_node_ids = node_ids->flatten();
+
+        // Reserve space for all the requested IDs and shrink afterwards
+        concrete_node_ids.reserve(selected_node_ids.size());
+        result.node_ranges.reserve(selected_node_ids.size());
+
+        for (const auto node_id : selected_node_ids) {
+            const auto it = node_ranges_.find(node_id);
+            if (it != node_ranges_.end()) {
+                update_layout(it->first, it->second);
+            }
+        }
+
+        concrete_node_ids.shrink_to_fit();
+        result.node_ranges.shrink_to_fit();
+    } else {
+        // node_ids Selection exists, but is empty
+    }
+
+    // Extract the ElementIDs from the GIDs
+    if (!result.node_ranges.empty()) {
+        const auto min = result.min_max_range.first;
+        const auto max = result.min_max_range.second;
+
+        std::vector<ElementID> element_ids;
+        auto dataset_elem_ids = pop_group_.getGroup("mapping").getDataSet("element_ids");
+        dataset_elem_ids.select({min}, {max - min}).read(element_ids);
+
+        result.ids.reserve(element_ids_count);
+
+        for (size_t i = 0; i < concrete_node_ids.size(); ++i) {
+            const auto node_id = concrete_node_ids[i];
+            const auto& range = result.node_ranges[i];
+
+            for (auto i = (range.first - min); i < (range.second - min); i++) {
+                result.ids.emplace_back(make_key<T>(node_id, element_ids[i]));
+            }
+        }
+
+        // Temp. fix: When you ask for a large hyperslab in a dataset and then move
+        //            to another dataset in the same file where you also ask for
+        //            another large range, the next IOps take an extra few seconds.
+        //            We observed that fooling HDF5 hides the issue, but we should
+        //            verify this behaviour once new releases of HDF5 are available.
+        dataset_elem_ids.select({min}, {1}).read(element_ids);
+    }
+
+    return result;
+}
+
 template <typename T>
 std::pair<size_t, size_t> ReportReader<T>::Population::getIndex(
     const nonstd::optional<double>& tstart, const nonstd::optional<double>& tstop) const {
@@ -314,11 +398,16 @@ std::pair<size_t, size_t> ReportReader<T>::Population::getIndex(
 
 
 template <typename T>
-DataFrame<T> ReportReader<T>::Population::get(const nonstd::optional<Selection>& selection,
+typename DataFrame<T>::DataType ReportReader<T>::Population::getNodeIdElementIdMapping(
+    const nonstd::optional<Selection>& node_ids) const {
+    return getNodeIdElementLayout(node_ids).ids;
+}
+
+template <typename T>
+DataFrame<T> ReportReader<T>::Population::get(const nonstd::optional<Selection>& node_ids,
                                               const nonstd::optional<double>& tstart,
                                               const nonstd::optional<double>& tstop,
                                               const nonstd::optional<size_t>& tstride) const {
-    DataFrame<T> data_frame;
     size_t index_start = 0;
     size_t index_stop = 0;
     std::tie(index_start, index_stop) = getIndex(tstart, tstop);
@@ -330,55 +419,25 @@ DataFrame<T> ReportReader<T>::Population::get(const nonstd::optional<Selection>&
         throw SonataError("tstart should be <= to tstop");
     }
 
+    // min and max offsets of the node_ids requested are calculated
+    // to reduce the amount of IO that is brought to memory
+    const auto node_id_element_layout = getNodeIdElementLayout(node_ids);
+    const auto& node_ranges = node_id_element_layout.node_ranges;
+    const auto min = node_id_element_layout.min_max_range.first;
+    const auto max = node_id_element_layout.min_max_range.second;
+
+    if (node_id_element_layout.ids.empty()) {  // At the end no data available (wrong node_ids?)
+        return DataFrame<T>{{}, {}, {}};
+    }
+
+    // Fill times
+    DataFrame<T> data_frame;
     for (size_t i = index_start; i <= index_stop; i += stride) {
         data_frame.times.push_back(times_index_[i].second);
     }
 
-    // Simplify selection
-    // We should remove duplicates
-    // And when we can work with ranges let's sort them
-    // auto nodes_ids_ = Selection::fromValues(node_ids.flatten().sort());
-    Selection::Values node_ids;
-
-    if (!selection) {  // Take all nodes in this case
-        node_ids.reserve(nodes_pointers_.size());
-        std::transform(nodes_pointers_.begin(),
-                       nodes_pointers_.end(),
-                       std::back_inserter(node_ids),
-                       [](const std::pair<NodeID, Range>& node_pointer) {
-                           return node_pointer.first;
-                       });
-    } else if (selection->empty()) {
-        return DataFrame<T>{{}, {}, {}};
-    } else {
-        node_ids = selection->flatten();
-    }
-
-    Ranges positions;
-    // min and max offsets of the node_ids requested are calculated
-    // to reduce the amount of IO that is brought to memory
-    uint64_t min = std::numeric_limits<uint64_t>::max();
-    uint64_t max = std::numeric_limits<uint64_t>::min();
-    auto dataset_elem_ids = pop_group_.getGroup("mapping").getDataSet("element_ids");
-    for (const auto& node_id : node_ids) {
-        const auto it = nodes_pointers_.find(node_id);
-        if (it == nodes_pointers_.end()) {
-            continue;
-        }
-        min = std::min(it->second.first, min);
-        max = std::max(it->second.second, max);
-        positions.emplace_back(it->second.first, it->second.second);
-
-        std::vector<ElementID> element_ids(it->second.second - it->second.first);
-        dataset_elem_ids.select({it->second.first}, {it->second.second - it->second.first})
-            .read(element_ids.data());
-        for (const auto& elem : element_ids) {
-            data_frame.ids.push_back(make_key<T>(node_id, elem));
-        }
-    }
-    if (data_frame.ids.empty()) {  // At the end no data available (wrong node_ids?)
-        return DataFrame<T>{{}, {}, {}};
-    }
+    // Fill ids
+    data_frame.ids = node_id_element_layout.ids;
 
     // Fill .data member
     size_t n_time_entries = ((index_stop - index_start) / stride) + 1;
@@ -392,6 +451,7 @@ DataFrame<T> ReportReader<T>::Population::get(const nonstd::optional<Selection>&
             fmt::format("DataType of dataset 'data' should be Float32 ('{}' was found)",
                         dataset_type.string()));
     }
+
     std::vector<float> buffer(max - min);
     for (size_t timer_index = index_start; timer_index <= index_stop; timer_index += stride) {
         // Note: The code assumes that the file is chunked by rows and not by columns
@@ -401,25 +461,26 @@ DataFrame<T> ReportReader<T>::Population::get(const nonstd::optional<Selection>&
         off_t offset = 0;
         off_t data_offset = (timer_index - index_start) / stride;
         auto data_ptr = &data_frame.data[data_offset * n_ids];
-        for (const auto& position : positions) {
-            uint64_t elements_per_gid = position.second - position.first;
-            uint64_t gid_start = position.first - min;
+        for (const auto& range : node_ranges) {
+            uint64_t elements_per_gid = range.second - range.first;
+            uint64_t gid_start = range.first - min;
 
             // Soma report
             if (elements_per_gid == 1) {
                 data_ptr[offset] = buffer[gid_start];
             } else {  // Elements report
-                uint64_t gid_end = position.second - min;
+                uint64_t gid_end = range.second - min;
                 std::copy(&buffer[gid_start], &buffer[gid_end], &data_ptr[offset]);
             }
             offset += elements_per_gid;
         }
     }
+
     return data_frame;
 }
 
 template class ReportReader<NodeID>;
-template class ReportReader<std::pair<NodeID, ElementID>>;
+template class ReportReader<CompartmentID>;
 
 }  // namespace sonata
 }  // namespace bbp
